### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    #! format: off
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
    #! format: on
end

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
begin
	using HypertextLiteral, PlutoUI, ColorSchemes, PlutoPlotly,  SparseArrays, PlotlyKaleido
	using CSV, DataFrames, Dates, LinearAlgebra, NLsolve , StatsBase, CommonMark
end

# ╔═╡ e6a2c6d1-4b78-429f-9647-81bd4e7a644b
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""
# Week 2 - Introduction to Macroeconomics

## Exercises

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas**

**February 2025**
"""

# ╔═╡ c2968c3b-d631-4c4d-a28c-4949c17795dd
md"""
---
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""
### Packages used in this notebook
"""

# ╔═╡ 47ae8d5c-0284-4b00-80ae-2ced7ccb61ef
md"""
---
"""

# ╔═╡ 5bad5b8f-1d72-4a8c-b971-8ad2afa62a67
md"""
## Exercise 1. Endogenous vs Exogenous Variables (I)
"""

# ╔═╡ e6e49172-2ff2-4f21-900a-ba6a829d341c
md"""

Consider the production of corn on an isolated island in a given year. The population has 1 ton of corn seeds left from the previous harvest to be sowed this year. There is a water reservoir with 100 million liters of water available, but using this water for irrigation requires labor effort. Considering the available amount of seeds, the water in the reservoir, the labor effort, and the amount of corn produced:

👉 a) Which ones would you consider to be the endogenous variables? 

👉 b) Which ones would be the exogenous variables?

👉 c) Can you conceive other exogenous that may be variables associated with this activity?
"""

# ╔═╡ c5e8c978-8fac-4012-b4c9-8e74f5aca223
md"""
!!! tip "Answer (a)"

	The endogenous variables are (i) the amount of corn produced and (ii) the labor effort allocated to the irrigation process.
	
"""

# ╔═╡ cc6ab8ad-6b05-4256-bd24-a0819fbd2f96
md"""
!!! tip "Answer (b)"

	The exogenous variables are (i) the amount of available seeds and (ii) the amount of water available in the reservoir.

"""

# ╔═╡ 4d4660f6-e802-4d46-96cb-fb920ecb73b5
md"""
!!! tip "Answer (c)"

	Three other exogenous variables that will certainly affect the two endogenous variables are (i) the quantity of rain that will fall, (ii) the quality of the soil, and (iii) the occurrence of plagues.

"""

# ╔═╡ 9acf52b5-5557-46e0-8dde-78e42c462c28
md"""
---
"""

# ╔═╡ dcd13560-0f73-4aaa-b59a-b7961d8cdc63
md"""
## Exercise 2. Endogenous vs Exogenous Variables (II)
"""

# ╔═╡ df9417d2-ebe1-498c-b299-f2b8fdee1084
md"""
*Adapted (with changes) from the textbook*

Sciences other than economics also use models to explain the behavior of endogenous variables based on assumptions about the environment and changes in exogenous variables. Suppose you have to design a model to explain childhood obesity and its links to diabetes and some other factors, like genetic conditions, food habits, and sedentarism.

👉 **a)** Which ones would you consider to be the endogenous variables? 

👉 **b)** Which ones would be the exogenous variables?

👉 **c)** Can you think of other exogenous variables?
"""

# ╔═╡ cec1d956-027e-445e-9b9e-db2aacfe71b1
md"""
!!! tip "Answer (a)"

	1. Apparently, we would be inclined to think that, in this case, obesity would be the exogenous variable and diabetes the endogenous one. However, carefully considering these two variables will tell us a different story. 

	2. Note that not all obese children have diabetes, and not all children that have diabetes are obese, so obesity can not be an exogenous variable *de per se*.

	3. Therefore, diabetes and childhood obesity are both endogenous variables. Both are the result of something else.
	
"""

# ╔═╡ d4a82d4d-c8de-4a25-88e3-aed876d1968a
md"""
!!! tip "Answer (b)"

	Exogenous variables that explain both childhood obesity and diabetes are food habits, genetic conditions, and sedentarism.

"""

# ╔═╡ a262c4d6-7303-41f0-a18e-6950079d4bfc
md"""
!!! tip "Answer (c)"

	Other exogenous variables that explain both childhood obesity and diabetes are e.g. education, income level, advertising, and geographic locations.

"""

# ╔═╡ 902604c3-155b-4f95-9d50-b11c133b836a
md"""
---
"""

# ╔═╡ 9e05bc6e-a423-41c2-a236-41432276af88
md"""
## Exercise 3. Solving a Macroeconomic Model
"""

# ╔═╡ 5222cb8c-363c-4fbd-b9a9-ec9553d4c8ab
md"""

Slide 6 (week 2) displays a graphical representation of an artificial macroeconomic model (its figure can be found below). In this model, we know the impact each variable will have on the other variables (given by the number attached to each arrow) and we also know that $X_1 = 10, X_2 = 5$.

"""

# ╔═╡ 94ac2cde-d8f2-47f9-b27f-e9828fe6722c
Resource("http://modernmacro.org/resources/Vivaldo/figures/model.png", :width=>500)

# ╔═╡ 09426867-1b5e-45c0-8e88-b81832e99bc9
md"""
👉 a) Which ones are the exogenous variables? And Why?
"""

# ╔═╡ e3ff7943-d008-49f6-882d-f911da21e6d8
md"""
!!! tip "Answer (a)"

	Exogenous variables are: $X_1,X_2$ because we know their values _a priori_: $X_1=10, X_2=5$.

"""

# ╔═╡ 33898360-dfa8-4a63-b122-ad9ed71cc987
md"""
👉 b) Which ones are the endogenous variables? And Why?
"""

# ╔═╡ f1ba33fa-e242-424d-9891-51d5e4d9addd
md"""
!!! tip "Answer (b)"

	Endogenous variables are $Q_a,Q_b,Q_c,Q_d$. They are the unknowns in the model. They are determined by the exogenous variables and by the structure of the model (explained in (d))

"""

# ╔═╡ cd17d2f5-af54-4a06-a2de-d5b750e05f31
md"""
👉 c) How many parameters do we have in this model?
"""

# ╔═╡ 36b68560-0ca1-4248-bdfc-4b9f448c0f52
md"""
!!! tip "Answer (c)"

	Parameters are 8 altogether: $2,8, 5, 0.5, 0.1, ...$. They give the impact of one variable upon another variable (provided by each arrow in the figure above).

"""

# ╔═╡ 71f8374a-7f2e-41a5-a713-bfdad0309990
md"""
👉 d) What is the structure of the model?
"""

# ╔═╡ 33ec3e63-9b6e-48cb-b7dd-c98ac5878e2a
md"""
!!! tip "Answer (d)"

	The model's structure transforms inputs into outputs and is given by the set of parameters.

"""

# ╔═╡ 3cf179b7-db29-4f93-99ee-abe2d348aa75
md"""
👉 e) Solve numerically the model represented in the figure above.
"""

# ╔═╡ 53d6dc4d-b4cc-48e3-8b6e-fb0ca690dc81
md"""
We can solve this model using linear algebra or the NLsolve package. The linear algebra solution can be found in Exercise 10. We recommend the use of NLsolve because it is simple and more intuitive. As we will see, the syntax makes the original model and its computational counterpart exactly the same.
"""

# ╔═╡ 935f5b31-a293-46d1-ba7c-83637ce6a6b7
md"""
#### Solution using NLsolve
"""

# ╔═╡ e1117af1-2a87-44ab-adac-f99844d144e0
md"""
The model in the figure above can be written down as:

$$\begin{aligned}
Q_ b &=2 Q_{a}+8 X_{2} \\
Q_{a} &=0.1 Q_{d}-0.5 Q_{c} \\
Q_{d} &=0.5 Q_{c} \\
Q_{c} &=1.5 Q_{a}+2.5 Q_{b}+5 X_{1}
    \end{aligned}$$
"""

# ╔═╡ 525e50f7-a253-476d-99e9-fa2fe670de42
md"""
First, NLsolve requires the model to be written as a homogeneous system, which in this case looks like:


$$\begin{aligned}
&0=2 Q_a+8 X_2 -Q_b\\
&0 = 0.1 Q_{d} -0.5 Q_{c}-Q_{a} \\
&0=0.5 Q_c -Q_d\\
&0=1.5 Q_a+2.5 Q_b+5 X_1 -Q_c
\end{aligned}$$

"""

# ╔═╡ e5151b0e-8f56-487e-89f2-b27b940d3ad8
md"""
Then we have to write down our problem according to the syntax of NLsolve: 

- give a name to our problem: let us name it as _zazu_

- define the variables in our problem: v[1] , v[2] , v[3] , v[4] 

- define the equations in our problem: F[1] , F[2] , F[3] , F[4]

Finally, we should compute the solution to our problem according to the syntax of NLsolve: 

- give a name to the solution to our problem: let us name it as _solution _ zazu_

- call the NLsolve function that will solve the problem: _nlsolve_ 

- tell the problem to be solved (in this case it is _zazu_), 

- give four initial guesses for the package to start the computation: in linear problems $[0.0 ; 0.0 ; 0.0 ; 0.0]$ always works.

- the number of initial guesses must equal the number of unknowns in the model.
"""

# ╔═╡ 1d6983c1-dfca-4c1a-be8e-fe63313212c5
md"""
!!! note
	Using the next cell, and considering that the values of X1 and X2 are associated with the sliders below, provide a numerical solution to this problem.
"""

# ╔═╡ 31b69cd9-3cea-4064-9d4a-e9a154d86336
md"""
---
"""

# ╔═╡ c7d27ed4-69b4-4e67-9dce-4323ddea4b59
md"""
## Exercise 4. Plotting Real GDP
"""

# ╔═╡ 0328e243-0b07-4480-8d9d-36d321287766
md"""
Using data from the CONFERENCE BOARD, ["The  Total Economy Database"](https://www.conference-board.org/data/economydatabase/total-economy-database-productivity), we plot the Real GDP levels for Portugal and France.

👉 Can we get much information about these plots regarding the performance of the two economies? Which country performed better? Which one showed more volatility in GDP?
"""

# ╔═╡ f33600d1-a1d9-41e8-b9b3-4107de25df54
md"""
!!! answer "Suggested answer"

	1. Simple plots like these do not provide much relevant information about each economy's performance. We need a more refined analysis, such as growth rates, the amplitude and persistence of business cycles, and cross-correlation indicators.

	2. Students usually portray plots like these to provide relevant information about one country (or one variable), hoping their actions are helpful. In most cases, plotting the original time series of any major macroeconomic variable that displays a trend like these two here is of little interest in itself.

"""

# ╔═╡ de1a9561-f0c0-4046-b38b-8fae087e583e
md"""

---

"""

# ╔═╡ 3af6b135-74d4-45f2-aa2e-aab24a5307ed
md"""
## Exercise 5. Business Cycles
"""

# ╔═╡ 42f371eb-ff16-4017-8deb-e53d92e85b2e
md"""
In the various plots below, we present the following:

1. The evolution of Real GDP and Real Potential GDP for Portugal and France;

2. The business cycles as the percentage deviation of Real GDP from Potential GDP for Portugal, France, and the UK.

3. The coss-correlation of business cycles in Europe.

Based on the information provided by these plots, answer the following questions.

"""

# ╔═╡ 6134353f-870b-44fb-83b2-a6bc02d04d8a
md"""

👉  **a)** By looking at the evolution of Real GDP and Potential GDP, can you spot one period where each economy was in a recession? And in an economic boom?

"""

# ╔═╡ f4dab38f-262f-421f-b3a9-a5a4759b95c8
md"""
!!! answer "Answer (a)"

	1. An economic boom occurs when Real GDP is higher than Potential GDP, and a recession occurs when the opposite occurs. 

	2. Therefore, by zooming in on the plot above, it is not difficult to see that the Portuguese economy was in a boom in 1973 while it was under a recession in 1975.

	3. For France, if we zoom in on the plot, we can easily spot that the same type of cycles happened in the same periods.

"""

# ╔═╡ 9dfbe21a-6ba0-4f50-8554-67a5b9150aad
md"""
👉 **b)** The following plot displays the business cycles in Portugal, France, and the UK. What main points can we observe by inspecting this figure?
"""

# ╔═╡ 8a6e9b90-dd85-4aa3-8d60-64bac56c3886
#savefig(p2_9, "Cycles_France.svg", width=900)

# ╔═╡ cd9202f0-939e-4356-9560-a12540169fe2
md"""
!!! answer "Answer (b)"

	There are four main points we can observe in the plot above: 

	- The cycles of those three economies tend to move together over time.

	- The amplitude or volatility of the cycles (distance from top to bottom) seems to be relatively similar over time, with two exceptions: 2019/20 (Covid) and 1974/75 (oil crisis). 

	- The persistence of the cycles (for how long they take place) seems similar across these economies.

	- Cycles are recurrent phenomena. It suggests that shocks hit these economies regularly, and sometimes those shocks seem very large (like in 1974/75 and 2019/20).


"""

# ╔═╡ 560e72fe-f816-41f3-9d01-e7d0ef4a8cb4
md"""

👉 **c)** By looking at the correlation matrix of the business cycles in the OECD, which is presented below, mention three highly positively correlated countries and three with cycles that display low or no correlation at all.
"""

# ╔═╡ 38d51c87-9de4-4e09-9d90-c136b5f40cdc
md"""
!!! note "What is a correlation matrix?"

    - A correlation matrix plots the cross-correlation coefficients between a set of variables. In our case here, the variables are the business cycles of a set of countries (JPN for Japan, KOR for Korea, PRT for Portugal, and so on).  

    - The cross-correlation coefficient is a statistical measure of the similarity of two series over time. It assumes values between $-1$ and $1$. Between the business cycles of Portugal and France, the cross-correlation is equal to 0.6879 (see cell below). This value means that if France is going into an economic boom/recession, there is a likelihood of 68.79% that Portugal will follow France. 

	Example: the cross-correlation between Portugal and France is equal to $+0.6879.$ Put the mouse on top of the cell common to Portugal and France, and you will get this value.

	- A column with many blue entries means the business cycles in this country are not highly correlated with those in other countries. On the other hand, the more red the cells become, the higher the cross-correlation will be.


"""

# ╔═╡ 6f6b5a36-05c9-4624-bc6a-135e5b6d7ba6
md"""
!!! answer "Answer (c)"

	New Zealand, Korea, and Taiwan have the lowest cross-correlation of their cycles with the rest of this group. In Europe, Norway, Spain, and Greece also have low cross-correlation.

	In the case of the European zone (located on the left low corner of the heatmap), for example, Belgium, France, and the UK (GBR) have cycles displaying the highest cross-correlation with the other countries in this zone.
"""

# ╔═╡ 52332845-62e4-4115-ae10-3828db0c0048
md"""
---
"""

# ╔═╡ 98ffd713-b851-4bca-8e20-99e21cd42561
md"""
## Exercise 6: The Impact of Business Cycles
"""

# ╔═╡ a2641d70-f16c-45e2-9aa6-6a880a9c2bad
md"""
In 2007, the world economy was struck by a financial crisis of enormous proportions. This crisis affected some countries more than others in the European Union and affected more some zones of the world economy than others. Let us see what has happened to 23 economies since 2007. For that purpose, we should normalize Real GDP levels for all 23 economies: Real GDP is set to 100 in 2007.

We apply such procedure to three groups of countries:

- The EuroZone periphery countries
- The EuroZone core countries + US, Canada, Switzerland and Japan
- South Eastern Asian countries +  Australia and New Zealand

After looking at the three plots that you will find in this exercise, rank the three groups of countries according to their macroeconomic performance after the impact of the financial crisis. What is your opinion about what is happening in the case of Greece?
"""

# ╔═╡ 5a240502-4e3d-4dae-a53b-5428c21e0237
md"""
!!! hint

	Look at how many years it took the various countries to get back to the level of real GDP they had when the crisis hit them (2007).
"""

# ╔═╡ 134a3f92-1475-4c1d-ad7d-bf00ba9291d7
md"""
!!! answer "Suggested answer"

	From the three previous plots, we can put forward four main points:

	1. The EuroZone periphery was the group that suffered most from the financial crisis of 2007; in fact, there are countries in this group that, 15 years on, their GDP levels are still below those of 2007.

	2. The EuroZone core, plus most other Western countries (like the US, Canada, Switzerland, and others), took only seven years to recover their 2007 GDP levels; this group did not suffer that much from this crisis.

	3. The South East Asian countries, Australia and New Zealand were unaffected by this financial crisis. Japan is the exception, which may be explained by the fact that the Japanese economy is much more aligned with Western economies than with China, as the others are.

	4. The case of Greece is a remarkable negative example of the functioning of the EU and the EuroZone. A democratic and open society at the heart of the EU, how can it lose 25% of its 2007 GDP after 15 years? It seems that something is rotten in Europe.

"""

# ╔═╡ 44cfea34-cb41-4387-bf38-bf5030bfcf0b
md"""
---
"""

# ╔═╡ add07024-b7eb-445d-b4f4-6a6757c2ecd3
md"""
## Exercise 7. Inflation vs Interest Rates 
"""

# ╔═╡ bfb890be-fba4-4d55-9e39-a52d40a5adad
md"""

In macroeconomics, one of the most important data types is quarterly data. It presents the data relevant to analyzing short-term variations in economic activity. We collected data for three critical macroeconomic variables:

1. The unemployment rate: UR
2. The inflation rate as the percentage change of the Consumer Price Index: CPI
3. The Fed Funds Rate as the interest rate set by the US central bank (called the Federal Reserve Board or simply **Fed**): FFR 

"""

# ╔═╡ 3df87bac-8e0f-4a9a-8c98-69b96cee9f6c
md"""
👉 a) The following figure plots the evolution of the rate of inflation (CPI) and the Federal Funds Rate (FFR). Can we obtain relevant information about the relationship between these two variables simply by looking at the two time series?
"""

# ╔═╡ 6bab0554-e0c2-4f82-bd73-f14ef8e7ecc3
md"""
!!! answer "Answer (a)"

	Time series usually provide very crude information, and we have the same problem here. We can say, for example, that the FFR is generally greater than the CPI, suggesting that the real interest rate is usually positive (we will cover the concept of a "real interest rate" better next week). We can see also that the inflation rate can be negative, while the FFR has been bounded above zero in the USA. 

"""

# ╔═╡ 2b8c2477-cf94-4784-86c0-cd2b59984034
md"""
👉 b) The following figure presents a scatter plot (also known as a cross-plot) between the rate of inflation (CPI) and the Federal Funds Rate (FFR). Can we obtain relevant information about the relationship between these two variables by looking at the cross-plot?
"""

# ╔═╡ d18ab447-8a4b-47f3-aee1-39c3c97d8135
md"""
!!! answer "Answer (b)"

	The cross-plot above refines the two-time series and, for this reason, provides more useful information. In this case, we can easily see a clear positive relationship between those two macroeconomic variables. That is, on average, when the inflation rate increases, the FFR also goes up. Therefore, we get significant evidence that the Fed increases the FFR when inflation rises.

"""

# ╔═╡ 4f812277-8f34-4d5c-aa32-57d65e173f62
md"""
---
"""

# ╔═╡ 9727dff3-5300-4ffe-9fcb-28c19accd77e
md"""
## Exercise 8. Inflation vs Unemployment 
"""

# ╔═╡ c7a08720-2e79-47e0-8895-da38047b2acf
md"""
👉 a) The following two figures plot the evolution of the rate of inflation (CPI) and the unemployment rate (UR). What kind of information can we get about the relationship between these two macroeconomic variables by inspecting these plots?
"""

# ╔═╡ 43da915a-3bb0-424d-b841-221f0fd64575
md"""
!!! hint

	The cross-correlation coefficient between these two variables equals $+0.064$, as we can check in the following cell. This coefficient ranges from $-1$ to $+1$. Therefore, a value of $0.064$ indicates an insignificant correlation.

"""

# ╔═╡ 18ee1822-e759-4944-9a67-326bb3f3ded2
md"""
!!! answer "Suggested answer"

	From the two previous plots, we can put forward two main points:

	1. It appears that there is little correlation between the rate of inflation and the rate of unemployment, considering the whole period. The correlation coefficient is close to zero: $0.062$, confirming the lack of correlation.

	2. It is clear that large loops exist between these variables, suggesting that some other variables must influence these two variables asymmetrically.

"""

# ╔═╡ 8273fb1a-ff8a-4b8b-92d8-26b2b44ebbbe
md"""
---
"""

# ╔═╡ 4c9e3c52-771a-4601-9dbc-0c267f432c2f
md"""
## Exercise 9. The Budget Balance
"""

# ╔═╡ 17ff14f3-a546-4783-b9a7-0183cb4716fd
md"""
In the following plot, we present the evolution of another crucial macroeconomic variable: the budget balance as a percentage of GDP. The evidence covers the US economy from 1929 to 2023. This variable is very important because it provides some information about the sustainability of public finances and the government's intervention in managing booms and recessions.
"""

# ╔═╡ 7a93f8d0-81dc-43d6-b4de-56d113af3ea1
md"""
👉 a)  Based on the evidence in the figure above, what is your opinion concerning that macroeconomic variable's volatility (variability)? **_Hint. The standard deviation equals 4.72%._**
"""

# ╔═╡ e0b5a2d3-2779-46d9-ba6a-8097ea6d5f59
md"""
!!! answer "Answer (a)"

	It is easy to see that the US public budget as a percentage of GDP is very volatile over time. Apart from the 1950s and 1960s, when its values were small and relatively stable, the situation was entirely different for the entire period. For the whole period, its standard deviation is close to 4.72%, reflecting a large volatility for this variable type.
"""

# ╔═╡ b5c2d16e-4e32-4926-9635-5bed7af0718b
md"""
👉 b)  In a period that comprises almost 100 years, how often did the USA experience positive budget balances?
"""

# ╔═╡ a7012952-8ce8-4473-b798-96bf680279d4
md"""
!!! answer "Answer (b)"

	In a period of 95 years altogether, only in 14 years did the US economy end up with a positive budget balance as a percentage of GDP. Even in this case, the levels of those superhavits were tiny compared to the deficits.
"""

# ╔═╡ 4a086be3-a7e9-4c0d-88b5-3b1109725e44
md"""
👉 c)  You are starting to study macroeconomics. If you were asked, **_at this stage_**, what do you think is the level of public debt as a percentage of GDP in the US by now, given that the mean of the public budget has been equal to $-3.3\%$ of GDP since 1929, what would you answer?
"""

# ╔═╡ 396ffb70-ec3b-44bc-88e7-4c02b9a3a20d
md"""
!!! answer "Answer (c)"

	A layman's answer will be something like this:

	>"If a country runs public deficits every year, it means that such country is living beyond its means. Its public debt will grow without bound, and sooner or later, this country will face bankruptcy".

	_Note. You will learn in week 11 that such a layman's view is wrong despite looking wise and common sense. It is for reasons like this one that learning a bit of macroeconomics is good for you._

"""

# ╔═╡ a2e363f1-780c-4355-9e03-99e281aeaab1
md"""
---
"""

# ╔═╡ 5bfd068e-f755-4733-892f-7e6cb0f6d547
md"""
## Exercise 10. International Oil Prices
"""

# ╔═╡ ef2e0e3e-56cc-4ef0-a67d-4ef261b86b2b
md"""
Oil and gas are still the largest components of energy consumption. From an economic point of view, almost everything we do depends on burning some form of energy. As those two sources are still the predominant ones today, a significant change in their prices will dramatically affect production costs and inflation. The following figure presents evidence of the annual growth rates of international oil prices since the 1950s. 
"""

# ╔═╡ 4de5e69e-8d0f-4e8a-aa70-02129b2d15ff
md"""
👉 **a)** How would you describe the evolution of international oil prices since 1973?
"""

# ╔═╡ 48e0e26f-63b3-4f7c-97f1-c74516fce3a6
md"""
!!! answer "Answer (a)"

	International oil prices were stable until 1973. After that year, they displayed an astonishing characteristic: their growth rate jumped up and down in a huge range, somewhere between $+120\%$ and $-50\%$ per year, in a systematic process year after year.

"""

# ╔═╡ 8fb22f8c-c461-47c2-9c97-94c08002c042
md"""
👉 **b)** Economists call these jumps in oil prices "temporary shocks" that affect inflation. Why do you think these shocks are called temporary instead of permanent shocks?
"""

# ╔═╡ a745b953-a92c-4c7c-a38a-86795f83b135
md"""
!!! answer "Answer (b)"

	They are called "shocks" because individual economies can not do anything to prevent them: they just hit the economy, and there is very little we can do about it in the short term.

	They are called "temporary" because the growth rates of oil prices may go up (and even a lot), but soon they will go down, even in the short term. If the shock were permanent, oil prices would have to display a growth rate that had to remain relatively constant.

"""

# ╔═╡ 08009f68-a92d-4135-83b2-99a5aa4f160c
md"""
---
"""

# ╔═╡ 5773b804-b89c-4254-9bbf-1759eff1f0ba
md"""
## Exercise 11. Rules vs Discretion
"""

# ╔═╡ ca46c228-8321-4f79-85e0-05a2c4fc000d
md"""
*From the textbook*

"*Consider the difficult task of raising children. One of the most widely recognized challenges of this task is to properly balance rules and ad-hoc decisions. Constantly breaking rules might send the wrong message to a kid, while strictly enforcing rules every time might result in excessive punishments. The debate about the conduct of macroeconomic policy is not significantly different from this example.*"

👉 **a)** Comment on the American Recovery and Reinvestment Act 2009 (ARRA 2009). Can this Act be characterized as discretionary policy?

👉 **b)** Is it possible for this set of policies to affect the incentives of financial intermediaries or other major economic agents?
"""

# ╔═╡ 2040bb61-81f9-4c16-8ba3-9ec419dbce31
md"""
!!! hint
    "The approximate cost of the ARRA 2009 economic stimulus package was estimated to be $787 billion at the time of passage, later revised to \$831 billion between 2009 and 2019", from [Wikipedia] (https://en.wikipedia.org/wiki/American_Recovery_and_Reinvestment_Act_of_2009). In fact, given that the level of Real GDP of the US in 2008 was \$15752.3 billion, \$831 billion over ten years corresponds to less than a 0.5% increase in public spending *per year*. Looking at these numbers and comparing them with what happened recently with the COVID-19 relief plan (an increase of 6 trillion dollars in two years), the numbers of the ARRA 2009 stimulus package look ridiculously irrelevant.
"""

# ╔═╡ 5b1860bb-30e7-485d-b643-30222c8f4ce0
md"""
!!! answer "Suggested answer (a)"

	Yes, the ARRA 2009 can be considered as discretionary policy. While there is no well-accepted rule to conduct fiscal policy, it is common sense that when there is a recession like that in 2008/2010, the government should step in and boost public spending to counterbalance the dramatic reduction of private expenditure. So, from this point of view, the ARRA program is not a completely ad-hoc measure. Everybody was expecting it. It looks like an implicit, wise, and flexible rule for fiscal policy.

"""

# ╔═╡ febd8e59-417f-4190-9814-9115abef99f9
md"""
!!! answer "Suggested answer (b)"

	The ARRA 2009 did not represent an irresponsible increase in public spending of such a magnitude that it would undermine the confidence in the sustainability of US public debt. Therefore, one can not envision a way for such a program to have negatively affected financial intermediaries and other economic agents; precisely, the opposite.

"""

# ╔═╡ 49313d24-84e6-43a8-9e31-040d84e6bf62
md"""
---
"""

# ╔═╡ 1ccaf9c4-c972-4877-bfa8-a49cc126ca6a
md"""
## Exercise 12. Solving exercise 3 with linear algebra (optional)
"""

# ╔═╡ a414e917-e746-434f-b26e-3dfff16bd9a1
md"""
We solved exercise 2 above using NLsolve. However, we could have used Linear Algebra for the same task. The two methods deliver the same result. We recommend that students in this course use NLsolve, but if one wants to have a go with the use of algebra, that is perfectly acceptable.
"""

# ╔═╡ 6bd27e4a-0f32-45ca-9b2a-4e2eecb22071
md"""
The model in the slides can be written down as:

$$\begin{aligned}
Q_ b &=2 Q_{a}+8 X_{2} \\
Q_{a} &=0.1 Q_{d}-0.5 Q_{c} \\
Q_{d} &=0.5 Q_{c} \\
Q_{c} &=1.5 Q_{a}+2.5 Q_{b}+5 X_{1}
    \end{aligned}$$

Solving the model above by substitution is quite an exasperating task. There is a much more accessible way to solve this model: use **linear algebra**. We can write the model as follows (we use decimal places to make visual inspection easier):

$$\begin{aligned}
-2.0Q_{a} + 1.0Q_{b} + 0.0 Q_{c} + 0.0 Q_{d} & = 0.0 X_{1}+8.0 X_{2} \\
1.0 Q_{a}  +  0.0 Q_{b}+ 0.5 Q_{c}-0.1 Q_{d} & = 0.0 X_{1}+0.0 X_{2} \\
0.0 Q_{a} +  0.0 Q_ b - 0.5 Q_{c}+1.0 Q_{d} & = 0.0 X_{1}+0.0 X_{2} \\
-1.5 Q_{a} - 2.5 Q_{b}+1.0 Q_{c}+0.0 Q_{d}& = 5.0 X_{1}+0.0 X_{2}
    \end{aligned}$$


Let us organize the equations above using matrices:

$$\underbrace{\left[\begin{array}{cccc}
-2.0 & 1.0 & 0.0 & 0.0 \\
1.0 & 0.0 & 0.5 & -0.1 \\
0.0 & 0.0 & -0.5 & 1.0 \\
-1.5 & -2.5 & 1.0 & 0.0
\end{array}\right]}_{A} \cdot \underbrace{\left[\begin{array}{c}
Q_{a} \\
Q_{b} \\
Q_{c} \\
Q_{d}
\end{array}\right]}_{Q} = \ \underbrace{\left[\begin{array}{cc}
0.0 & 8.0 \\
0.0 & 0.0 \\
0.0 & 0.0 \\
5.0 & 0.0
\end{array}\right]}_{B} \cdot \underbrace{\left[\begin{array}{l}
X_{1} \\
X_{2} \\
\end{array}\right]}_{X} \tag{1}
$$

so, we can write

$$A \cdot Q=B \cdot X \tag{2}$$

or

$$Q = \underbrace{(A^{-1}\cdot B}_{C}) \ X \tag{3}$$

where $A^{-1}$ is called the inverse matrix of $A$ (extremely easy to calculate using a computer).

It is interesting that by using linear algebra, we can easily distinguish the elements of our model:


- "$Q$" is the output of the model (endogenous variables)

- "$X$" is the input of the model (exogenous variables)

- "$C$" is the structure of the model (parameters)
"""

# ╔═╡ f35219aa-0dc5-4e92-8ce9-85b1bfa72998
md"""
Now let us pass the four matrices into the notebook.
"""

# ╔═╡ f9e5a05d-1f88-4579-9d28-596f3bd672e0
A= [-2.0  1.0  0.0  0.0 ; 1.0  0.0  0.5 -0.1 ; 0.0  0.0 -0.5  1.0 ; -1.5  -2.5  1.0  0.0]

# ╔═╡ ff46bc55-750c-4ea9-a934-3b01cf046a8d
inv(A)

# ╔═╡ a4e3d715-8e01-48de-8127-3edd6bd278c0
B = [0.0  8.0 ;  0.0  0.0 ; 0.0  0.0 ; 5.0  0.0]


# ╔═╡ 4877135b-3d4b-43c2-a34e-f1ed68955406
C = inv(A)*B 				# inv(A) is the inverse matrix of A

# ╔═╡ 89cbc406-e188-4e59-98ed-1fd255d9a260
md"""
---
"""

# ╔═╡ 437c44ae-bb54-4b31-975b-9924eb1e30cc
md"""
## Auxiliary cells (do not remove them)
"""

# ╔═╡ 8b7e9add-97a4-46e2-8fc9-89fcfa83f1f5
md"""
##### Data files used in this notebook

- OECD\_Major\_Aggregates.csv
- DataUS.csv
- US\_BudgetDeficits\_1929\_2023.csv
- Hutchins-fiscal-impact.csv
- Spread.csv
- Oil_prices.csv

These csv files must be kept in the same folder as this notebook.
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ 450ba864-6fdf-4e59-80c3-bbf236568865
#PlotlyKaleido.start() ; force_pluto_mathjax_local(true) ; PlotlyKaleido.start(;mathjax = true)

# ╔═╡ 434a6918-806d-476d-8d8b-de8607429385
md"### CSS code"

# ╔═╡ 9c545455-9631-49f1-b9a7-8c3c03b39920
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1600px;
    		padding-left: max(100px, 10%);
    		padding-right: max(380px, 10%); 
            # 383px to accommodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ dde9a9f6-0013-401e-9081-8ef5d599d8ca
begin
      struct TwoColumns{L, R}
    left::L
    right::R
      end
      
      function Base.show(io, mime::MIME"text/html", tc::TwoColumns)
          write(io, """<div style="display: flex;"><div style="flex: 20%;margin-left:19%;">""")
          show(io, mime, tc.left)
          write(io, """</div><div style="flex: 20%;">""")
          show(io, mime, tc.right)
          write(io, """</div></div>""")
      end
end

# ╔═╡ 0ce3b90b-d2ab-4019-b8f3-5a9c59ef1885
TwoColumns(
md"""

`X1 = `$(@bind X1 Slider(0.0:2.5:20.0, default=10.0, show_value=true))

""",
	
md"""

`X2 = `$(@bind X2 Slider(0.0:2.5:20.0, default=5.0, show_value=true))

"""
)

# ╔═╡ 20e7c708-ba16-40c5-a888-fbc98e2e0671
begin
	# definig the problem
	function zazu(F, v)
		
			# defining variables
           	Qa = v[1]
           	Qb = v[2]
		   	Qc = v[3]
			Qd = v[4]

			# defining equations
           	F[1] = 2Qa + 8X2 - Qb
           	F[2] = 0.1Qd - 0.5Qc - Qa
			F[3] = 0.5Qc - Qd
			F[4] = 1.5Qa + 2.5Qb + 5X1 - Qc
	end

	#getting the solution
	solution_zazu = nlsolve(zazu, [0.0 ; 0.0 ; 0.0 ; 0.0])
	solution_zazu.zero
end

# ╔═╡ 1feff701-2a76-494e-a822-935bb436986f
X = [X1 ; X2]

# ╔═╡ bf22ae24-4cdd-460b-8664-ffbd62568e15
Q = C*X


# ╔═╡ d4d46822-9db3-4adc-a6c7-692b97062931
cm"""
!!! answer "Answer (e)"
	Therefore, the solution for the model is given by:
	
	> 	Qa = $(round.((Q[1,1]), digits=4)) &nbsp;  , &nbsp;  Qb = $(round.((Q[2,1]), digits=4)) &nbsp; ,  &nbsp; Qc = $(round.((Q[3,1]), digits=4))  &nbsp; , &nbsp;  Qd = $(round.((Q[4,1]), digits=4))	

	Notice that if we change the sliders above, these values will change accordingly.

"""

# ╔═╡ b6cc10ab-a6c2-4337-8fc8-9430083924e2
cm"""
!!! answer "Answer"
	Therefore, the solution for the model is given by:
	
	> 	Qa = $(round.((Q[1,1]), digits=4)) &nbsp;  , &nbsp;  Qb = $(round.((Q[2,1]), digits=4)) &nbsp; ,  &nbsp; Qc = $(round.((Q[3,1]), digits=4))  &nbsp; , &nbsp;  Qd = $(round.((Q[4,1]), digits=4))	

	Notice that if we change the sliders above, these values will change accordingly.

"""

# ╔═╡ f6ba4955-6ee2-4503-bf28-d47cacef4b9d
md"### Exercises"

# ╔═╡ 5614ff13-77c0-4589-9232-c078b7d85154
md"""
##### Supporting cells for exercise 4 (Plotting real GDP)
"""

# ╔═╡ e5dad7e7-68e4-4a47-854a-4ad61ba34f7a
mydata = CSV.read("OECD_MajorAggreg.csv", DataFrame);

# ╔═╡ 00c5aa04-41de-43b1-b825-421fdbf01fe9
begin
	trace2_1 = scatter(;x = mydata.YEARS,  y = mydata.PRT, name = "Real GDP", line_color = "Blue")

	layout2_1 = Layout(;
					height = 450,
					title_text = "Real GDP: Portugal (1950-2021)",
					title_x = 0.5,
					hovermode = "x",
        			xaxis_title = "Years",
        			#xaxis_range = [1960, 2020],
        			yaxis_title = "Millions of 2020 international dollars",
        			#yaxis_range=[-2, 2],
        			titlefont_size = 16)

	p2_1 = plot([trace2_1], layout2_1)
	#savefig(p0, "Portugal_Real_GDP.pdf")
end


# ╔═╡ 4b150ec7-fddc-4d82-83e7-3032a1368833
begin
	trace2_2 = scatter(;x = mydata.YEARS, y = mydata.FRA, name = "Real GDP", line_color = "Maroon")

	layout2_2 = Layout(;
					height = 450,
					title_text = "Real GDP: France (1950-2021)",
					title_x = 0.5,
					hovermode = "x",
        			xaxis_title = "Years",
        			#xaxis_range = [1960, 2020],
        			yaxis_title = "Millions of 2020 international dollars",
        			#yaxis_range=[-2, 2],
        			titlefont_size = 16)

	p2_2 = plot([trace2_2], layout2_2)
	#savefig(p1_2, "Portugal_Real_GDP.pdf")
end

# ╔═╡ e12d354f-3e41-4925-8ac7-e0b9f2203d27
md"""
##### Supporting cells for exercise 5 (Business cycles)
"""

# ╔═╡ 24dcd4c4-8655-4c1a-b86f-580c28c2ac49
log_mydata = log.(mydata[:,2:end]);

# ╔═╡ 4ef49779-bbe2-448d-8f65-5dc7642c2fb1
names(mydata);

# ╔═╡ 1f9c7f64-bd8d-4d26-89ea-bce5bf697247
begin
	function hp_filter(log_mydata::Array{Float64,1}, lambda::Float64)
    	n = length(log_mydata)
    	@assert n >= 4
    	diag2 = lambda*ones(n-2)
    	diag1 = [ -2lambda; -4lambda*ones(n-3); -2lambda ]
    	diag0 = [ 1+lambda; 1+5lambda; (1+6lambda)*ones(n-4); 1+5lambda; 1+lambda ]
    	D = spdiagm(-2 => diag2, -1 => diag1, 0 => diag0, 1 => diag1, 2 => diag2)
    	D\log_mydata
	end
end;

# ╔═╡ 991002ef-4999-43eb-9370-dccc77eabea0
begin
	hp_trend = zeros(72,23)
	for n = 1:23
    	hp_trend[:,n] = hp_filter(log_mydata[:,n], 7.0)
	end
	hp_trend
end;

# ╔═╡ 478501bf-5bc3-482d-aae0-ebbf972ff616
begin

	trace2_3 = scatter(;x = mydata.YEARS, y = log_mydata[:,19], name = "Real GDP", line_color = "Blue")
	trace2_4 = scatter(;x = mydata.YEARS, y = hp_trend[:,19], name = "Potential GDP", line_color = "Red")

	layout2_4 = Layout(;
		height = 450,
		title_text = "Real vs Potential Real GDP: Portugal (1950-2021)",
		title_x = 0.5,
		hovermode = "x",
        xaxis_title = "Years",
        #xaxis_range = [1960, 2020],
        yaxis_title = "Log of Real GDP",
        #yaxis_range=[-2, 2],
        titlefont_size = 16)

	p2_4 = plot([trace2_3 , trace2_4], layout2_4)

end

# ╔═╡ 4cf851a1-f517-41ce-a97b-df43815bb95c
# For France
begin

	trace2_5 = scatter(;x = mydata.YEARS, y = log_mydata[:,12], name = "Real GDP", line_color = "Blue")
	trace2_6 = scatter(;x = mydata.YEARS, y = hp_trend[:,12], name = "Potential GDP", line_color = "Red")

	layout2_6 = Layout(; 
		height = 450,
		title_text = "Real vs Potential Real GDP: France (1950-2021)",
		title_x = 0.5,
		hovermode = "x",
        xaxis_title = "Years",
        #xaxis_range = [1960, 2020],
        yaxis_title = "Log of Real GDP",
        #yaxis_range=[-2, 2],
        titlefont_size = 16)

	p2_6 = plot([trace2_5 , trace2_6], layout2_6)

end

# ╔═╡ 4e75e662-f1af-43f0-ac1e-30459c1d0814
countries = names(log_mydata);

# ╔═╡ 32326d0c-7be1-40e5-aac8-ee60ab34e418
begin
	cycles = zeros(72,23)
	for n = 1:23
    	cycles[:,n] = log_mydata[:,n] - hp_trend[:,n]
	end
	cycles
end;

# ╔═╡ 75b3c457-246b-4509-8ff2-b00f0de38c19
begin
	trace2_7 = scatter(;x = mydata.YEARS, y = cycles[:,19] , name = "Portugal", line_color = "Blue",
         			mode="markers+lines", marker_size=6, marker_symbol="circle", line_width=0.3)

	trace2_8 = scatter(;x = mydata.YEARS, y = cycles[:,12], name = "France", line_color = "Red",
         			mode="markers+lines", marker_size=7.5, marker_symbol="star", line_width=0.5)

	trace2_9 = scatter(;x = mydata.YEARS, y = cycles[:,23], name = "UK", line_color = "Gray",
         			mode="markers+lines", marker_size=6, marker_symbol="o", line_width=0.3)

	layout2_9 = Layout(;
					height = 450,
					title_text = "Business cycles: Portugal, France, and UK (1950-2021)",
					#title_text = "Business cycles: France (1950-2021)",
					title_x = 0.5,
					hovermode = "x",
        			xaxis_title = "Years",
        			xaxis_range = [1949, 2022],
        			yaxis_title = "% deviations from Potential GDP",
        			#yaxis_range=[-2, 2],
        			titlefont_size = 16)

	p2_9 = plot([trace2_7, trace2_8, trace2_9], layout2_9)
	
	add_shape!(p2_9, line( x0=1949, y0=0.0, x1=2022, y1=0.0,  line=attr(color="RoyalBlue", width=1.5, dash="dashdot")))
	p2_9
	#savefig(p2_9, "G:\\Teaching\\Master\\Slides\\01_Introduction\\fig2022\\cyclesV2.pdf", width = 1000)
end

# ╔═╡ be6cccff-4731-4eeb-9134-1f98b5942cc0
cross_corr_matrix = crosscor(cycles[:,:],cycles[:,:], [0]; demean=true);

# ╔═╡ 59014736-a141-423b-8f27-243aadbc08b6
heat_matrix = reshape(cross_corr_matrix, 23, 23);

# ╔═╡ 72d20281-7a95-49d3-b217-7ebbc73f4814
let
	function heatmap1()
    x_labels = ["JPN", "KOR", "TWN", "CAN", "USA", "AUS", "NZL", "AUT", "BEL", "DNK", "FIN", "FRA", "DEU", "GRC", "IRL", "ITA", "NLD", "NOR", "PRT", "ESP", "SWE", "CHE", "GBR"]
    y_labels = reverse(x_labels)
    
    # Assuming heat_matrix is a 2D array with the same order as x_labels and y_labels
    reversed_heat_matrix = reverse(reverse(heat_matrix, dims=1), dims=2)
    
    trace = heatmap(
        x=reverse(x_labels),
        y=y_labels,
        z=reversed_heat_matrix,
        colorscale="RdBu"
    )
    
    plot(trace, Layout(title_text="Cross-Correlation Heatmap of the business cycles in the OECD (1950-2021)", width=650, height=650))
end

heatmap1()
end

# ╔═╡ e022b698-e9cf-44c4-aeb4-8ad2245ed5aa
begin
	function heatmap2()
    trace = heatmap(
			
        x=["AUT", "BEL", "DNK", "FIN", "FRA", "DEU", "GRC", "IRL", "ITA", "NLD", "NOR", "PRT", "ESP", "SWE", 								"CHE", "GBR"],
        y=["AUT", "BEL", "DNK", "FIN", "FRA", "DEU", "GRC", "IRL", "ITA", "NLD", "NOR", "PRT", "ESP", "SWE", 								"CHE", "GBR"],
        z= heat_matrix[8:end,8:end], colorscale="RdBu"
    				)
    Plot(trace, 
		Layout(title_text= " Correlation matrix of the business cycles in Europe: 1950-2021", 
			title_x = 0.5, width=650, height=600))
	end
	heatmap2()
	#savefig(heatmap2(), "correl_matrix.svg")
end;

# ╔═╡ 910927a2-47ae-40ff-b8cd-73ef2b5a93aa
md"""
##### Supporting cells for exercise 6 (The impact of business cycles)
"""

# ╔═╡ 842d4b3e-8b8e-4c44-9a32-56f40307df41
data2007 = mydata[58:72,1:24];

# ╔═╡ 25ada4af-7a6c-47ec-87b6-facd3ffd3e64
begin
	PRT_1 = data2007[1,20] * 100 / data2007[1,20] # Fixing the initial level to 100
	PRT_2 = data2007[2,20] * 100 / data2007[1,20]
	PRT_3 = data2007[3,20] * 100 / data2007[1,20]
	PRT_2
end;

# ╔═╡ 9fe70f93-faca-4ea9-917b-8e594d78cdc3
begin
	t = 15;
	PRT1 = data2007[1,20] * 100 / data2007[1,20] 		# Fixing the initial value to 100
	PRTn = length(PRT1) 								# the number of initial conditions (1 in this case)
	PRTt = [PRT1  zeros(PRTn, t-1)]; 					# Preallocating memory for the output

		for i = 1 : t-1
    		PRTt[i+1] = data2007[i+1,20] * 100 / data2007[1,20];
		end
	PRTt
end;

# ╔═╡ 1eb1551d-491d-4955-9d2f-853a0971fb0c
begin
	data2007_2 = DataFrame([(c ./ first(c))*100
        for c ∈ eachcol(data2007[:,2:end])], names(data2007[:,2:end]))
end;

# ╔═╡ 4932c69b-6536-4067-9d87-6b1bfe34cac8
begin
	data1 = AbstractTrace[]
	##################################
	col_idx1 = [11, 14, 15, 16, 19, 20]
	some_colnames1 = []
		for k in col_idx1
    		push!(some_colnames1, names(data2007_2)[k])
		end
	##################################
		for col in some_colnames1
    		push!(data1, scatter(x = 2007:2021, y = data2007_2[!,col], name = col, mode = "markers+lines",
            		marker_size = 7, marker_symbol = "circle", line_width = 0.5 ))
		end

	layout2_11 = Layout(;
			height = 450,
			title_text = "The impact of the financial crisis in the EZ Periphery (Real GDP)", 
			title_x = 0.5,
			colorway=ColorSchemes.phase.colors[1:40:end],
			hovermode="x",		
        	xaxis_title = "Years",
        	xaxis_range = [2006.75, 2021.25],
        	yaxis_title = "GDP Index",
        	#yaxis_range=[-2, 2],
        	titlefont_size=16)

	p2_11 = plot(data1, layout2_11)
	add_shape!(p2_11, line( x0=2007, y0=100, x1=2022, y1=100,  line=attr(color="RoyalBlue", width=1.5, dash="dashdot")))
	p2_11
end

# ╔═╡ f2776549-3b8f-4a11-b500-53276c252fce
begin
	data2 = AbstractTrace[]
	#################
	col_idx2 = [1, 4, 5, 8, 9, 10, 12, 13, 17, 18, 21, 22, 23]
	some_colnames2 = []
		for k in col_idx2
    		push!(some_colnames2, names(data2007_2)[k])
		end
#################
		for col in some_colnames2
    		push!(data2, scatter(x = 2007:2021, y = data2007_2[!,col], name = col, mode = "markers+lines",
            	marker_size = 7, marker_symbol = "circle", line_width = 0.5 ))
		end

	layout2_12 = Layout(;
				height = 450,
				#colorway=ColorSchemes.phase.colors[1:20:end],
				title_text = "The impact of the financial crisis in the EZ-Core+ (Real GDP)", 
				title_x=0.5,
				hovermode="x",
				#colorway=ColorSchemes.phase.colors[1:25:end],
        		xaxis_title = "Years",
        		xaxis_range = [2006.75, 2021.25],
        		yaxis_title = "GDP Index",
        		#yaxis_range=[-2, 2],
        		titlefont_size=16)

	p2_12 = plot(data2, layout2_12)
	add_shape!(p2_12, line( x0=2007, y0=100, x1=2022, y1=100,  line=attr(color="RoyalBlue", width=1.5, dash="dashdot")))
	p2_12
end

# ╔═╡ b2fa9b87-0387-4c1a-baba-cd756568d4d7
begin
	data3 = AbstractTrace[]
	#################
	col_idx3 = [1, 2, 3, 6, 7]
	some_colnames3 = []
		for k in col_idx3
    		push!(some_colnames3, names(data2007_2)[k])
		end
#################
		for col in some_colnames3
    		push!(data3, scatter(x = 2007:2021, y = data2007_2[!,col], name = col, mode = "markers+lines",
            	marker_size = 7, marker_symbol = "circle", line_width = 0.5 ))
		end

	layout2_13 = Layout(;
				height = 450,
				title_text = "The impact of the financial crisis in Asia + Australasia (Real GDP)", 	  					title_x=0.5,
				hovermode="x",		
        		xaxis_title = "Years",
        		xaxis_range = [2006.75, 2021.25],
				#xaxis_range = ["2006-10-01", "2021-06-30"],
        		yaxis_title = "GDP Index",
        		#yaxis_range=[-2, 2],
        		titlefont_size=16)

	

	p2_13 = plot(data3, layout2_13)
	
	add_shape!(p2_13, line( x0=2007, y0=100, x1=2022, y1=100,  line=attr(color="RoyalBlue", width=1.5, dash="dashdot")))
	p2_13
end

# ╔═╡ 6cc8c3bd-5619-480e-9454-e9c64bad8689
begin
	 layout2_10 = Layout(;
			title_text = "The impact of the financial crisis in Portugal (Real GDP)",
			title_x = 0.5,
			hovermode = "x",
        	xaxis_title = "Years",
        	#xaxis_range = [1960, 2020],
        	yaxis_title = "GDP Index",
        	#yaxis_range=[-2, 2],
        	titlefont_size=16)


	p2_10 = Plot(2007:2021,PRTt', layout2_10, name = "Portugal", line_color = "Blue",
         		mode="markers+lines", marker_size=7, marker_symbol="circle", line_width=0.3)
end;

# ╔═╡ 6d001dc9-5977-4b02-bcc2-1a2106ede495

md"""
##### Supporting cells for exercises 7 and 8 (Inflation, FFR, unemployment)
"""

# ╔═╡ 335c550f-fbd9-44e2-bc69-c13b54250a45
DataUS = CSV.read("DataUS.csv", DataFrame);

# ╔═╡ 40f34f88-3d33-4557-831e-22607f3682f7
begin
	trace1_14 = scatter(;x = DataUS.Quarters, y = DataUS.CPI, name = "CPI", line_color = "#1919c9",
						mode = "lines", line_width="2", marker_size ="5")
	trace1_15 = scatter(;x = DataUS.Quarters, y = DataUS.FFR, name = "FFR", line_color = "Maroon",
						mode = "lines", line_width="2", marker_size ="5")

	layout1_15 = Layout(;
				height = 450,
				title_text = "Inflation rate (CPI) versus Fed Funds Rate (FFR): USA (1960Q1--2024Q2)",
				title_x = 0.5,
				hovermode = "x",
	
            	axis = attr(
                	title = "Quarterly observations",
                	#tickformat = "%Y",
                	#hoverformat = "%Y-Q%q",
                	#tick0 = "1959/01/01",
               	 	#dtick = "M120"
				),
        		#xaxis_range = [Date.(1959), Date.(2024)],
				xaxis_range = ["1959-10-01", "2024-06-30"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2],
        		titlefont_size = 16)

	p1_15 = plot([trace1_14 , trace1_15], layout1_15)
end

# ╔═╡ 3aad82ff-baa4-48d5-aa66-51344c754421
begin
	layout1_16 = Layout(;
	height=550, width = 600,
					title_text = "Inflation (CPI) vs Fed Funds Rate (FFR) in the USA (1960Q1--2024Q2)",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			xaxis_range = [-1, 18],
        			xaxis_title = "Fed Funds Rate",
        			yaxis_range=[-3, 15],
        			titlefont_size=16)

	trace2 = scatter(; x = [-2, 20] , y = [-2, 20], line_width = 3 )

	p1_16 = plot(DataUS.FFR, DataUS.CPI, mode="markers+lines", text=DataUS.Quarters,
            	marker_symbol="circle", marker_size="6",line_width=0.0,
            	#marker_color="LightSteelBlue",
            	marker_color="Blue",layout1_16)
	addtraces!(p1_16, trace2)
	restyle!(p1_16, 1:2, name=["Rates", "45º"])
	p1_16
	#savefig(p1_16, "G:\\Teaching\\Master\\Slides\\01_Introduction\\fig2022\\costlyinflation.pdf", width = 1000)
end

# ╔═╡ 52b6eb6e-1566-4e0f-b787-404cdc5f9edb
begin
	trace2_17 = scatter(;x = DataUS.Quarters, y = DataUS.UR, name = "UR", line_color = "BlueViolet",
						mode = "lines", line_width="2", marker_size ="4.5")
	trace2_18 = scatter(;x = DataUS.Quarters, y = DataUS.CPI, name = "CPI", line_color = "DarkRed",
						mode = "lines", line_width="2", marker_size ="4.5")

	layout2_18 = Layout(;
				height = 450,
				title_text = "Inflation (CPI) versus Unemployment Rate (UR): USA (1960.Q1--2023.Q4)",
				title_x = 0.5,
				hovermode = "x",
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/01/01",
                dtick = "M120",
        				),
        		#xaxis_range = [Date.(1959), Date.(2024)],
				xaxis_range = ["1959-10-01", "2024-06-30"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2],
        		titlefont_size = 16)

	p2_18 = plot([trace2_17 , trace2_18], layout2_18)
end

# ╔═╡ 8ae18d1b-dcd1-43f4-828e-63dab4373091
begin
	layout2_19 = Layout(;
					height=550, width = 600,
					title_text = "Inflation (CPI) versus Unemployment Rate (UR): USA (1960Q1--2024Q2)",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Rate of unemployment",
        			#yaxis_range=[-2, 2],
        			titlefont_size=16)

	p2_19 = plot(DataUS.UR, DataUS.CPI, mode="markers+lines", text=DataUS.Quarters,
            	marker_symbol="circle", marker_size="6",line_width=0.3,
            	marker_color="#0248ff", #262896
            	#marker_color="DarkBlue",
				layout2_19)

end

# ╔═╡ 356367d5-9127-4a93-bda0-9c7ff1efe9f8
crosscor(DataUS.UR, DataUS.CPI, [0]; demean=true) # cross-correlation between UR and CPI

# ╔═╡ 04421307-25ad-4bb3-a686-caeedc56dd61
md"""
##### Supporting cells for exercise 9 (The budget balance)
"""

# ╔═╡ 3ecd4946-8bee-4ed1-84dd-1f172d3adef4
 USBudget = CSV.read("US_BudgetDeficits_1929_2023.csv", DataFrame);

# ╔═╡ ed446d9d-27be-4f0d-b7dd-5046174bc425
begin
	trace2_27 = bar(;x= USBudget[:,1], y= USBudget[:,2],
               marker=attr(color="Blue", opacity=0.5),
               name="BD US")

	layout2_27 = Layout(;
		height = 450,
		title_text = "Federal Budget as a % of GDP (USA: 1929-2023)",
		title_x = 0.5,
		hovermode = "x",
        xaxis_title = "Anual obervations",
        xaxis_range = [Date.(1927), Date.(2025)],
		#xaxis_range = ["2003-10-01", "2023-04-30"],
        separators = ".",
        yaxis_title = "% points",
        #yaxis_range=[-2, 2],
        titlefont_size = 16,
        bargap=0.25)
	p2_27 = plot(trace2_27, layout2_27)
end

# ╔═╡ 2807f3bd-4b64-437e-9e79-bfcbe9723e2a
#USDebt = CSV.read("US_PublicDebt.csv", DataFrame);

# ╔═╡ f906a80e-947e-4480-85f8-e4776ef9ea23
#g_vs_r= CSV.read("g_versus_r.csv", DataFrame);

# ╔═╡ 09c4faf0-6236-43c5-8d28-f9a0d2b506f7
md"""
##### Supporting cells for exercise 10 (International Oil prices)
"""

# ╔═╡ a2f584a0-b543-46b8-857e-266eb02eb351
Oil = CSV.read("Oil_prices.csv", DataFrame);

# ╔═╡ 687b864f-d431-4fbd-983b-8531ece3791c
begin
	p100=plot(Oil.Quarters, Oil.GrowthRate, line_color = "#1919c9", Layout(title_text = "Anual growth rates of international oil prices", title_x = 0.5, yaxis_title = "Percentage points", xaxis_title = "Quarterly observations", hovermode = "x"))
	
	add_shape!(p100, line( x0="1973-05-01", y0=-70.0, x1="1973-05-01", y1=200.0,
		line=attr(color="Maroon", width=1.5, dash="dashdot")))
	
	relayout!(p100,
		height = 450,
		annotations=[attr(		
		 x="1968-12-01", y=180, text="Yom Kippur war", showarrow=false, yshift=10,
		 font=attr( family="Calibri",  size=16,  color="Black" ) )	
		] )
	p100
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
ColorSchemes = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
CommonMark = "a80b9123-70ca-4bc0-993e-6e3bcb318db6"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
NLsolve = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"

[compat]
CSV = "~0.10.15"
ColorSchemes = "~3.27.0"
CommonMark = "~0.8.15"
DataFrames = "~1.7.0"
HypertextLiteral = "~0.9.5"
NLsolve = "~4.5.1"
PlotlyKaleido = "~2.2.5"
PlutoPlotly = "~0.6.2"
PlutoUI = "~0.7.60"
StatsBase = "~0.34.3"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "4af33cf20909f0ae45a95dcd93276d1175310421"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.Adapt]]
deps = ["LinearAlgebra", "Requires"]
git-tree-sha1 = "50c3c56a52972d78e8be9fd135bfb91c9574c140"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "4.1.1"

    [deps.Adapt.extensions]
    AdaptStaticArraysExt = "StaticArrays"

    [deps.Adapt.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.AliasTables]]
deps = ["PtrArrays", "Random"]
git-tree-sha1 = "9876e1e164b144ca45e9e3198d0b689cadfed9ff"
uuid = "66dad0bd-aa9a-41b7-9441-69ab47430ed8"
version = "1.1.3"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.ArrayInterface]]
deps = ["Adapt", "LinearAlgebra"]
git-tree-sha1 = "017fcb757f8e921fb44ee063a7aafe5f89b86dd1"
uuid = "4fba245c-0d91-5ea0-9b3e-6abc04ee57a9"
version = "7.18.0"

    [deps.ArrayInterface.extensions]
    ArrayInterfaceBandedMatricesExt = "BandedMatrices"
    ArrayInterfaceBlockBandedMatricesExt = "BlockBandedMatrices"
    ArrayInterfaceCUDAExt = "CUDA"
    ArrayInterfaceCUDSSExt = "CUDSS"
    ArrayInterfaceChainRulesCoreExt = "ChainRulesCore"
    ArrayInterfaceChainRulesExt = "ChainRules"
    ArrayInterfaceGPUArraysCoreExt = "GPUArraysCore"
    ArrayInterfaceReverseDiffExt = "ReverseDiff"
    ArrayInterfaceSparseArraysExt = "SparseArrays"
    ArrayInterfaceStaticArraysCoreExt = "StaticArraysCore"
    ArrayInterfaceTrackerExt = "Tracker"

    [deps.ArrayInterface.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    CUDA = "052768ef-5323-5732-b1bb-66c8b64840ba"
    CUDSS = "45b445bb-4962-46a0-9369-b4df9d0f772e"
    ChainRules = "082447d4-558c-5d27-93f4-14fc19e9eca2"
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    GPUArraysCore = "46192b85-c4d5-4398-a991-12ede77f4527"
    ReverseDiff = "37e2e3b7-166d-5795-8a7a-e32c996b4267"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArraysCore = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
    Tracker = "9f7883ad-71c0-57eb-9f7f-b5c9e6d3789c"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "bce6804e5e6044c6daab27bb533d1295e4a2e759"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.6"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "c785dfb1b3bfddd1da557e861b919819b82bbe5b"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.27.1"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"
weakdeps = ["SpecialFunctions"]

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.CommonMark]]
deps = ["Crayons", "PrecompileTools"]
git-tree-sha1 = "3faae67b8899797592335832fccf4b3c80bb04fa"
uuid = "a80b9123-70ca-4bc0-993e-6e3bcb318db6"
version = "0.8.15"

[[deps.CommonSubexpressions]]
deps = ["MacroTools"]
git-tree-sha1 = "cda2cfaebb4be89c9084adaca7dd7333369715c5"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.1"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.ConstructionBase]]
git-tree-sha1 = "76219f1ed5771adbb096743bff43fb5fdd4c1157"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.5.8"

    [deps.ConstructionBase.extensions]
    ConstructionBaseIntervalSetsExt = "IntervalSets"
    ConstructionBaseLinearAlgebraExt = "LinearAlgebra"
    ConstructionBaseStaticArraysExt = "StaticArrays"

    [deps.ConstructionBase.weakdeps]
    IntervalSets = "8197267c-284f-5f27-9208-e0e47529a953"
    LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "DataStructures", "Future", "InlineStrings", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrecompileTools", "PrettyTables", "Printf", "Random", "Reexport", "SentinelArrays", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "fb61b4812c49343d7ef0b533ba982c46021938a6"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.7.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "23163d55f885173722d1e4cf0f6110cdbaf7e272"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.15.1"

[[deps.Distances]]
deps = ["LinearAlgebra", "Statistics", "StatsAPI"]
git-tree-sha1 = "c7e3a542b999843086e2f29dac96a618c105be1d"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.12"

    [deps.Distances.extensions]
    DistancesChainRulesCoreExt = "ChainRulesCore"
    DistancesSparseArraysExt = "SparseArrays"

    [deps.Distances.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"
version = "1.11.0"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "7878ff7172a8e6beedd1dea14bd27c3c6340d361"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.22"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FiniteDiff]]
deps = ["ArrayInterface", "LinearAlgebra", "Setfield"]
git-tree-sha1 = "f089ab1f834470c525562030c8cfde4025d5e915"
uuid = "6a86dc24-6348-571c-b903-95158fe2bd41"
version = "2.27.0"

    [deps.FiniteDiff.extensions]
    FiniteDiffBandedMatricesExt = "BandedMatrices"
    FiniteDiffBlockBandedMatricesExt = "BlockBandedMatrices"
    FiniteDiffSparseArraysExt = "SparseArrays"
    FiniteDiffStaticArraysExt = "StaticArrays"

    [deps.FiniteDiff.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions"]
git-tree-sha1 = "a2df1b776752e3f344e5116c06d75a10436ab853"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.38"

    [deps.ForwardDiff.extensions]
    ForwardDiffStaticArraysExt = "StaticArrays"

    [deps.ForwardDiff.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.HashArrayMappedTries]]
git-tree-sha1 = "2eaa69a7cab70a52b9687c8bf950a5a93ec895ae"
uuid = "076d061b-32b6-4027-95e0-9a2c6f6d7e74"
version = "0.2.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "45521d31238e87ee9f9732561bfee12d4eebd52d"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.2"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IrrationalConstants]]
git-tree-sha1 = "e2222959fbc6c19554dc15174c81bf7bf3aa691c"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.4"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Artifacts", "Preferences"]
git-tree-sha1 = "a007feb38b422fbdab534406aeca1b86823cb4d6"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.7.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.Kaleido_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "43032da5832754f58d14a91ffbe86d5f176acda9"
uuid = "f7e6163d-2fa5-5f23-b69c-1db539e41963"
version = "0.2.1+0"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LineSearches]]
deps = ["LinearAlgebra", "NLSolversBase", "NaNMath", "Parameters", "Printf"]
git-tree-sha1 = "e4c3be53733db1051cc15ecf573b1042b3a712a1"
uuid = "d3d80556-e9d4-5f37-9878-2ab0fcc64255"
version = "7.3.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "13ca9e2586b89836fd20cccf56e57e2b9ae7f38f"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.29"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.MacroTools]]
git-tree-sha1 = "72aebe0b5051e5143a079a4685a46da330a40472"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.15"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NLSolversBase]]
deps = ["DiffResults", "Distributed", "FiniteDiff", "ForwardDiff"]
git-tree-sha1 = "a0b464d183da839699f4c79e7606d9d186ec172c"
uuid = "d41bc354-129a-5804-8e4c-c37616107c6c"
version = "7.8.3"

[[deps.NLsolve]]
deps = ["Distances", "LineSearches", "LinearAlgebra", "NLSolversBase", "Printf", "Reexport"]
git-tree-sha1 = "019f12e9a1a7880459d0173c182e6a99365d7ac1"
uuid = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
version = "4.5.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "cc0a5deefdb12ab3a096f00a6d42133af4560d71"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.1.2"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+2"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl"]
git-tree-sha1 = "1346c9208249809840c91b26703912dff463d335"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.6+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlotlyKaleido]]
deps = ["Artifacts", "Base64", "JSON", "Kaleido_jll"]
git-tree-sha1 = "ba551e47d7eac212864fdfea3bd07f30202b4a5b"
uuid = "f2990250-8cf9-495f-b13a-cce12b45703c"
version = "2.2.6"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Artifacts", "ColorSchemes", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "PrecompileTools", "Reexport", "ScopedValues", "Scratch", "TOML"]
git-tree-sha1 = "9ebe25fc4703d4112cc418834d5e4c9a4b29087d"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.6.2"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.PtrArrays]]
git-tree-sha1 = "1d36ef11a9aaf1e8b74dacc6a731dd1de8fd493d"
uuid = "43287f4e-b6f4-7ad1-bb20-aadabca52c3d"
version = "1.3.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.ScopedValues]]
deps = ["HashArrayMappedTries", "Logging"]
git-tree-sha1 = "1147f140b4c8ddab224c94efa9569fc23d63ab44"
uuid = "7e506255-f358-4e82-b7e4-beb19740aa63"
version = "1.3.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "3bac05bc7e74a75fd9cba4295cde4045d9fe2386"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.2.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "StaticArraysCore"]
git-tree-sha1 = "e2cc6d8c88613c05e1defb55170bf5ff211fbeac"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "1.1.1"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.11.0"

[[deps.SpecialFunctions]]
deps = ["IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "64cca0c26b4f31ba18f13f6c12af7c85f478cfde"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.5.0"

    [deps.SpecialFunctions.extensions]
    SpecialFunctionsChainRulesCoreExt = "ChainRulesCore"

    [deps.SpecialFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"

[[deps.StaticArraysCore]]
git-tree-sha1 = "192954ef1208c7019899fbf8049e717f92959682"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.3"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"
weakdeps = ["SparseArrays"]

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["AliasTables", "DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "29321314c920c26684834965ec2ce0dacc9cf8e5"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.4"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "a6b1675a536c5ad1a60e5a5153e1fee12eb146e3"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.0"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.7.0+0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─e6a2c6d1-4b78-429f-9647-81bd4e7a644b
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─c2968c3b-d631-4c4d-a28c-4949c17795dd
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─47ae8d5c-0284-4b00-80ae-2ced7ccb61ef
# ╟─5bad5b8f-1d72-4a8c-b971-8ad2afa62a67
# ╟─e6e49172-2ff2-4f21-900a-ba6a829d341c
# ╟─c5e8c978-8fac-4012-b4c9-8e74f5aca223
# ╟─cc6ab8ad-6b05-4256-bd24-a0819fbd2f96
# ╟─4d4660f6-e802-4d46-96cb-fb920ecb73b5
# ╟─9acf52b5-5557-46e0-8dde-78e42c462c28
# ╟─dcd13560-0f73-4aaa-b59a-b7961d8cdc63
# ╟─df9417d2-ebe1-498c-b299-f2b8fdee1084
# ╟─cec1d956-027e-445e-9b9e-db2aacfe71b1
# ╟─d4a82d4d-c8de-4a25-88e3-aed876d1968a
# ╟─a262c4d6-7303-41f0-a18e-6950079d4bfc
# ╟─902604c3-155b-4f95-9d50-b11c133b836a
# ╟─9e05bc6e-a423-41c2-a236-41432276af88
# ╟─5222cb8c-363c-4fbd-b9a9-ec9553d4c8ab
# ╟─94ac2cde-d8f2-47f9-b27f-e9828fe6722c
# ╟─09426867-1b5e-45c0-8e88-b81832e99bc9
# ╟─e3ff7943-d008-49f6-882d-f911da21e6d8
# ╟─33898360-dfa8-4a63-b122-ad9ed71cc987
# ╟─f1ba33fa-e242-424d-9891-51d5e4d9addd
# ╟─cd17d2f5-af54-4a06-a2de-d5b750e05f31
# ╟─36b68560-0ca1-4248-bdfc-4b9f448c0f52
# ╟─71f8374a-7f2e-41a5-a713-bfdad0309990
# ╟─33ec3e63-9b6e-48cb-b7dd-c98ac5878e2a
# ╟─3cf179b7-db29-4f93-99ee-abe2d348aa75
# ╟─53d6dc4d-b4cc-48e3-8b6e-fb0ca690dc81
# ╟─935f5b31-a293-46d1-ba7c-83637ce6a6b7
# ╟─e1117af1-2a87-44ab-adac-f99844d144e0
# ╟─525e50f7-a253-476d-99e9-fa2fe670de42
# ╟─e5151b0e-8f56-487e-89f2-b27b940d3ad8
# ╟─1d6983c1-dfca-4c1a-be8e-fe63313212c5
# ╟─0ce3b90b-d2ab-4019-b8f3-5a9c59ef1885
# ╠═20e7c708-ba16-40c5-a888-fbc98e2e0671
# ╟─d4d46822-9db3-4adc-a6c7-692b97062931
# ╟─31b69cd9-3cea-4064-9d4a-e9a154d86336
# ╟─c7d27ed4-69b4-4e67-9dce-4323ddea4b59
# ╟─0328e243-0b07-4480-8d9d-36d321287766
# ╟─00c5aa04-41de-43b1-b825-421fdbf01fe9
# ╟─4b150ec7-fddc-4d82-83e7-3032a1368833
# ╟─f33600d1-a1d9-41e8-b9b3-4107de25df54
# ╟─de1a9561-f0c0-4046-b38b-8fae087e583e
# ╟─3af6b135-74d4-45f2-aa2e-aab24a5307ed
# ╟─42f371eb-ff16-4017-8deb-e53d92e85b2e
# ╟─6134353f-870b-44fb-83b2-a6bc02d04d8a
# ╟─478501bf-5bc3-482d-aae0-ebbf972ff616
# ╟─4cf851a1-f517-41ce-a97b-df43815bb95c
# ╟─f4dab38f-262f-421f-b3a9-a5a4759b95c8
# ╟─9dfbe21a-6ba0-4f50-8554-67a5b9150aad
# ╟─75b3c457-246b-4509-8ff2-b00f0de38c19
# ╟─8a6e9b90-dd85-4aa3-8d60-64bac56c3886
# ╟─cd9202f0-939e-4356-9560-a12540169fe2
# ╟─560e72fe-f816-41f3-9d01-e7d0ef4a8cb4
# ╟─38d51c87-9de4-4e09-9d90-c136b5f40cdc
# ╟─72d20281-7a95-49d3-b217-7ebbc73f4814
# ╟─6f6b5a36-05c9-4624-bc6a-135e5b6d7ba6
# ╟─52332845-62e4-4115-ae10-3828db0c0048
# ╟─98ffd713-b851-4bca-8e20-99e21cd42561
# ╟─a2641d70-f16c-45e2-9aa6-6a880a9c2bad
# ╟─5a240502-4e3d-4dae-a53b-5428c21e0237
# ╟─4932c69b-6536-4067-9d87-6b1bfe34cac8
# ╟─f2776549-3b8f-4a11-b500-53276c252fce
# ╟─b2fa9b87-0387-4c1a-baba-cd756568d4d7
# ╟─134a3f92-1475-4c1d-ad7d-bf00ba9291d7
# ╟─44cfea34-cb41-4387-bf38-bf5030bfcf0b
# ╟─add07024-b7eb-445d-b4f4-6a6757c2ecd3
# ╟─bfb890be-fba4-4d55-9e39-a52d40a5adad
# ╟─3df87bac-8e0f-4a9a-8c98-69b96cee9f6c
# ╟─40f34f88-3d33-4557-831e-22607f3682f7
# ╟─6bab0554-e0c2-4f82-bd73-f14ef8e7ecc3
# ╟─2b8c2477-cf94-4784-86c0-cd2b59984034
# ╟─3aad82ff-baa4-48d5-aa66-51344c754421
# ╟─d18ab447-8a4b-47f3-aee1-39c3c97d8135
# ╟─4f812277-8f34-4d5c-aa32-57d65e173f62
# ╟─9727dff3-5300-4ffe-9fcb-28c19accd77e
# ╟─c7a08720-2e79-47e0-8895-da38047b2acf
# ╟─52b6eb6e-1566-4e0f-b787-404cdc5f9edb
# ╟─8ae18d1b-dcd1-43f4-828e-63dab4373091
# ╟─43da915a-3bb0-424d-b841-221f0fd64575
# ╠═356367d5-9127-4a93-bda0-9c7ff1efe9f8
# ╟─18ee1822-e759-4944-9a67-326bb3f3ded2
# ╟─8273fb1a-ff8a-4b8b-92d8-26b2b44ebbbe
# ╟─4c9e3c52-771a-4601-9dbc-0c267f432c2f
# ╟─17ff14f3-a546-4783-b9a7-0183cb4716fd
# ╟─ed446d9d-27be-4f0d-b7dd-5046174bc425
# ╟─7a93f8d0-81dc-43d6-b4de-56d113af3ea1
# ╟─e0b5a2d3-2779-46d9-ba6a-8097ea6d5f59
# ╟─b5c2d16e-4e32-4926-9635-5bed7af0718b
# ╟─a7012952-8ce8-4473-b798-96bf680279d4
# ╟─4a086be3-a7e9-4c0d-88b5-3b1109725e44
# ╟─396ffb70-ec3b-44bc-88e7-4c02b9a3a20d
# ╟─a2e363f1-780c-4355-9e03-99e281aeaab1
# ╟─5bfd068e-f755-4733-892f-7e6cb0f6d547
# ╟─ef2e0e3e-56cc-4ef0-a67d-4ef261b86b2b
# ╟─687b864f-d431-4fbd-983b-8531ece3791c
# ╟─4de5e69e-8d0f-4e8a-aa70-02129b2d15ff
# ╟─48e0e26f-63b3-4f7c-97f1-c74516fce3a6
# ╟─8fb22f8c-c461-47c2-9c97-94c08002c042
# ╟─a745b953-a92c-4c7c-a38a-86795f83b135
# ╟─08009f68-a92d-4135-83b2-99a5aa4f160c
# ╟─5773b804-b89c-4254-9bbf-1759eff1f0ba
# ╟─ca46c228-8321-4f79-85e0-05a2c4fc000d
# ╟─2040bb61-81f9-4c16-8ba3-9ec419dbce31
# ╟─5b1860bb-30e7-485d-b643-30222c8f4ce0
# ╟─febd8e59-417f-4190-9814-9115abef99f9
# ╟─49313d24-84e6-43a8-9e31-040d84e6bf62
# ╟─1ccaf9c4-c972-4877-bfa8-a49cc126ca6a
# ╟─a414e917-e746-434f-b26e-3dfff16bd9a1
# ╟─6bd27e4a-0f32-45ca-9b2a-4e2eecb22071
# ╟─f35219aa-0dc5-4e92-8ce9-85b1bfa72998
# ╠═f9e5a05d-1f88-4579-9d28-596f3bd672e0
# ╠═ff46bc55-750c-4ea9-a934-3b01cf046a8d
# ╠═a4e3d715-8e01-48de-8127-3edd6bd278c0
# ╠═4877135b-3d4b-43c2-a34e-f1ed68955406
# ╠═1feff701-2a76-494e-a822-935bb436986f
# ╠═bf22ae24-4cdd-460b-8664-ffbd62568e15
# ╟─b6cc10ab-a6c2-4337-8fc8-9430083924e2
# ╟─89cbc406-e188-4e59-98ed-1fd255d9a260
# ╟─437c44ae-bb54-4b31-975b-9924eb1e30cc
# ╟─8b7e9add-97a4-46e2-8fc9-89fcfa83f1f5
# ╟─9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─450ba864-6fdf-4e59-80c3-bbf236568865
# ╟─434a6918-806d-476d-8d8b-de8607429385
# ╟─9c545455-9631-49f1-b9a7-8c3c03b39920
# ╟─dde9a9f6-0013-401e-9081-8ef5d599d8ca
# ╟─f6ba4955-6ee2-4503-bf28-d47cacef4b9d
# ╟─5614ff13-77c0-4589-9232-c078b7d85154
# ╟─e5dad7e7-68e4-4a47-854a-4ad61ba34f7a
# ╟─e12d354f-3e41-4925-8ac7-e0b9f2203d27
# ╟─24dcd4c4-8655-4c1a-b86f-580c28c2ac49
# ╟─4ef49779-bbe2-448d-8f65-5dc7642c2fb1
# ╟─1f9c7f64-bd8d-4d26-89ea-bce5bf697247
# ╟─991002ef-4999-43eb-9370-dccc77eabea0
# ╟─4e75e662-f1af-43f0-ac1e-30459c1d0814
# ╟─32326d0c-7be1-40e5-aac8-ee60ab34e418
# ╟─be6cccff-4731-4eeb-9134-1f98b5942cc0
# ╟─59014736-a141-423b-8f27-243aadbc08b6
# ╟─e022b698-e9cf-44c4-aeb4-8ad2245ed5aa
# ╟─910927a2-47ae-40ff-b8cd-73ef2b5a93aa
# ╟─842d4b3e-8b8e-4c44-9a32-56f40307df41
# ╟─25ada4af-7a6c-47ec-87b6-facd3ffd3e64
# ╟─9fe70f93-faca-4ea9-917b-8e594d78cdc3
# ╟─1eb1551d-491d-4955-9d2f-853a0971fb0c
# ╟─6cc8c3bd-5619-480e-9454-e9c64bad8689
# ╟─6d001dc9-5977-4b02-bcc2-1a2106ede495
# ╟─335c550f-fbd9-44e2-bc69-c13b54250a45
# ╟─04421307-25ad-4bb3-a686-caeedc56dd61
# ╟─3ecd4946-8bee-4ed1-84dd-1f172d3adef4
# ╟─2807f3bd-4b64-437e-9e79-bfcbe9723e2a
# ╟─f906a80e-947e-4480-85f8-e4776ef9ea23
# ╟─09c4faf0-6236-43c5-8d28-f9a0d2b506f7
# ╟─a2f584a0-b543-46b8-857e-266eb02eb351
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
